package com.tianyi.sync.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.tianyi.sync.mapper.master.QueryMasterMapper;
import com.tianyi.sync.mapper.slave2.FallGroupSlaveMapper;
import com.tianyi.sync.model.DataSyncModel;
import com.tianyi.sync.model.GroupFallDataModel;
import com.tianyi.sync.model.ParmDataModel;
import com.tianyi.sync.service.GroupSyncService;
import com.tianyi.sync.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StopWatch;


import java.util.List;
import java.util.stream.Collectors;


@Service
public class GroupSyncServiceimpl implements GroupSyncService {

    private final static Logger logger = LoggerFactory.getLogger(GroupSyncServiceimpl.class);

    private final FallGroupSlaveMapper fallGroupSlaveMapper;

    private final QueryMasterMapper queryMasterMapper;

    public GroupSyncServiceimpl(QueryMasterMapper queryMasterMapper, FallGroupSlaveMapper fallGroupSlaveMapper) {
        this.queryMasterMapper = queryMasterMapper;
        this.fallGroupSlaveMapper = fallGroupSlaveMapper;
    }

    @Override
    @DSTransactional
    public Object groupSync() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        JSONObject jsonObject = new JSONObject();
        String lastMonth = DateUtils.getYearAndMonthLast();
        ParmDataModel parmDataModel = new ParmDataModel();
        parmDataModel.setPeriod(lastMonth);
//        parmDataModel.setPeriod("2024-06");
        String period = parmDataModel.getPeriod();
        parmDataModel.setYear(period.substring(0, 4));
        parmDataModel.setMonth(StringUtils.startsWith(period.substring(5, 7), "0") ? StringUtils.substringAfterLast(period.substring(5, 7), "0") : period.substring(5, 7));
        logger.info("参数:{}", JSON.toJSONString(parmDataModel));
        //先清除表内数据
        fallGroupSlaveMapper.delete(parmDataModel);
        //查询164库内数据
        List<DataSyncModel> dataSyncModelList = queryMasterMapper.queryGroupCurrentMonth(parmDataModel);
        if (CollectionUtils.isEmpty(dataSyncModelList)) {
            logger.info("unexpected event {}:",  "未查询到数据!");
            jsonObject.put("description", "未查询到数据!");
            return jsonObject;
        }
        List<GroupFallDataModel> groupFallDataModelList = dataSyncModelList.stream().map(v -> {
            GroupFallDataModel groupFallDataModel = new GroupFallDataModel();
            groupFallDataModel.setYear(v.getYear());
            groupFallDataModel.setMonth(v.getMonth());
            groupFallDataModel.setAccountcode(v.getAccount());
            groupFallDataModel.setEntitycode(v.getEntity());
            groupFallDataModel.setOrivalue(v.getMoney());
            groupFallDataModel.setICPcode(v.getICP());
            groupFallDataModel.setC1(v.getC1());
            groupFallDataModel.setC2(v.getC2());
            groupFallDataModel.setC3(v.getC3());
            groupFallDataModel.setC4(v.getC4());
            groupFallDataModel.setYtenantId("dmrn1j0i");
            return groupFallDataModel;
        }).collect(Collectors.toList());

        //数据落到96数据库内
        fallGroupSlaveMapper.addAll(groupFallDataModelList);

        jsonObject.put("msg", "数据同步成功!");
        jsonObject.put("code", 200);
        stopWatch.stop();
        logger.info("执行时间:{}", stopWatch.getLastTaskTimeMillis());
        return jsonObject;
    }


}
